--
-- ProfiLua
-- JTAG Scanner Execution Helper
--
-- (c) ASkr, www.askrprojects.net
--

gNumInputs  = 8
gNumOutputs = 4

InputNames  = {"ImSend","Addr","SendButs","PullAct","PullState","PowAct","PowState","ProbeAna"}
OutputNames = {"$SendStr","SendClk","Answ","PAna"}

function gInputName(channel)
	return InputNames[channel]
end

function gOutputName(channel)
	return OutputNames[channel]
end

function gSimStart(In)
	return {"",gHIGH,0,0}
end

function gSimStop(In)
end




-- clock operations:
--   0 - nothing
--   1 - string to output
--   2 - clock pulse low
--   3 - clock pulse high
--   4 - string to ""
clkOper = 0     
clkPin  = gHIGH -- state of clock PIN

sndButs = 0			-- last state of send buttons

answer  = gLOW  -- does this command trigger a standard, one byte answer?
pAna    = gLOW  -- analog probe output (6 bytes instead of only 1)

strSend = ""    -- string to send
strPrep = {}    -- table to send (prepared string content)


-- last states
pullAct  = 0
pullStat = 0
powAct   = 0
powState = 0
probeAna = 0




------------------------------------------------------------------------------
-- ImSend
--
-- Immediate send, send buttons are ignored, any change to Pull/Pow Active/State
-- will immediatly send a packet.
-- Additionally, the SYNC bit is discarded
------------------------------------------------------------------------------
function ImSend(In)

	-- abort if action in progrss
	if clkOper ~= 0 then
		return 0
	end
	
	-- changes on pullAct
	if In[4] ~= pullAct then
		pullAct = In[4]
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0000 -- start bit + address (shift left) + command
		if In[4] > 63 then
			In[4] = In[4] - 64
		end
		strPrep[2] = In[4]
		clkOper = 1

	-- changes on pullStat
	elseif In[5] ~= pullStat then
		pullStat = In[5]
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0001 -- start bit + address (shift left) + command
		if In[5] > 63 then
			In[5] = In[5] - 64
		end
		strPrep[2] = In[5]
		clkOper = 1
	
	-- changes on powAct
	elseif In[6] ~= powAct then
		powAct = In[6]
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0010 -- start bit + address (shift left) + command
		if In[6] > 63 then
			In[6] = In[6] - 64
		end
		strPrep[2] = In[6]
		clkOper = 1
	
	-- changes on powState
	elseif In[7] ~= powStat then
		powStat = In[7]
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0011 -- start bit + address (shift left) + command
		if In[7] > 63 then
			In[7] = In[7] - 64
		end
		strPrep[2] = In[7]
		clkOper = 1

	-- changes on powState
	elseif In[8] ~= probeAna then
		probeAna = In[8]
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0100 -- start bit + address (shift left) + command
		strPrep[2] = In[8]
		clkOper = 1
		
	end

	answer = gLOW
	pAna   = gLOW

	return clkOper
	
end




------------------------------------------------------------------------------
-- RegSend
--
-- Regular send.
------------------------------------------------------------------------------
function RegSend(In)
	-- order of the buttons (IN[3]) and bit weights:
	--   2  PullAct
	--   4  PullState
	--   8  PowAct
	--  16  PowState
	--  32  ReadDig
	--  64  SyncProbe
	-- 128  ProbeAnalog
	-- 256  ReadAnalog

	-- abort if action in progrss
	if clkOper ~= 0 then
		return 0
	end

	-- wait for button to be relased before proceeding with another command
	if sndButs > 0 then
		if In[3] > 0 then
			return 0
		else
			sndButs = 0
			return 0
		end
	end
	
	sndButs = In[3]
	answer = gLOW
	pAna = gLOW
	
	-- PullAct
	if In[3] == 2 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0000 -- start bit + address (shift left) + command
		strPrep[2] = In[4]
		clkOper = 1

	-- PullState
	elseif In[3] == 4 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0001 -- start bit + address (shift left) + command
		strPrep[2] = In[5]
		clkOper = 1

	-- PowAct
	elseif In[3] == 8 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0010 -- start bit + address (shift left) + command
		strPrep[2] = In[6]
		clkOper = 1

	-- PowState
	elseif In[3] == 16 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0011 -- start bit + address (shift left) + command
		strPrep[2] = In[7]
		clkOper = 1

	-- ReadDig
	elseif In[3] == 32 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b1100 -- start bit + address (shift left) + command
		clkOper = 1
		answer = gHIGH

	-- SyncProbe
	elseif In[3] == 64 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0110 -- start bit + address (shift left) + command
		clkOper = 1

	-- ProbeAnalog
	elseif In[3] == 128 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b0100 -- start bit + address (shift left) + command
		strPrep[2] = In[8]
		clkOper = 1
		
	-- ReadAnalog
	elseif In[3] == 256 then
		strPrep = {}
		strPrep[1] = 0x80 + In[2]*16 + 0b1101 -- start bit + address (shift left) + command
		clkOper = 1
		pAna = gHIGH -- 6 bytes output marker

	end

	return clkOper
	
end



------------------------------------------------------------------------------
-- Prep2String
--
--
------------------------------------------------------------------------------
function Prep2String(tab)
	local i
	local str = ""
	
	-- not that fast (due to ".."), but the string is not that long...
	for _,i in ipairs(tab) do
		if i == 0x10 then
			str = str..string.char(0x10)..string.char(0x10)
		elseif i == 0x00 then
			str = str..string.char(0x10)..string.char(0xff)
		else
			str = str..string.char(i)
		end
	end
	
	return str
	
end


------------------------------------------------------------------------------
-- gCalulate
--
--
------------------------------------------------------------------------------
function gCalculate(In)
	local Sent = 0

	if clkOper ~= 0 then

		-- clear answer pin
		answer = gLOW
		aAna   = gLOW
	
		-- string to output
		if clkOper == 1 then
			strSend = Prep2String(strPrep)
		
		-- clock pulse low
		elseif clkOper == 2 then
			clkPin = gLOW
		
		-- clock pulse high
		elseif clkOper == 3 then
			clkPin = gHIGH

		-- remove string from output
		elseif clkOper == 4 then
			strSend = ""
		end

		clkOper = clkOper + 1
		
		if clkOper > 4 then
			clkOper = 0
		end

		return {strSend,clkPin,answer,pAna}   -- *** RETURN
		
	end -- if clkOper ~= 0


	
	if In[1] >= gISHIGH then
		Sent = ImSend(In)
	end
	
	if Sent == 0 then
		Sent = RegSend(In)
	end

	return {strSend,clkPin,answer,pAna}     -- *** RETURN
end

